/*******************************************************************************

    RX_CR_TOM_5C VERSO 2

    C/ PIC 12F675


    RECEPTOR DE CONTROLE REMOTO POR TOM COM 5 CANAIS

    ACIONA OS CANAIS CONFORME OS TONS RECEBIDOS NA SUA ENTRADA.
    
    FUNCIONA COM MAIS DE UM BOTO ACIONADO AO MESMO TEMPO.
    
    SADAS C/ OPO P/ MODO 'RETENO' OU 'PULSO'.
    
    TRABALHA EM CONJUNTO COM 'TX_CR_TOM_5C VERSO 2'

    AUTOR: CLAUDIO LARIOS

    DATA:18/09/2020

    INFORMAO IMPORTANTE: TANTO O PIC DO TX COMO DO RX DEVEM TER SEUS VALORES
    DE CALIBRAO DO OSCILADOR INTERNO ORIGINAIS. ISTO  IMPORTANTE POIS TODA
    A TEMPORIZAO  BASEADA NA PREMISSA QUE O OSCILADOR INTERNO EST GERANDO
    4 MHZ.
    CASO TENHA ALTERADO ESTES VALORES NO IR FUNCIONAR. USE PIC'S NOVOS OU FAA
    A CALIBRAO DO VALOR DO BYTE 'OSCALL' NA LTIMA POSIO DA FLASH (0X3FF).
    
    OBS: FREQUNCIA DE ENTRADA TEM QUE TER CICLO POSITIVO IGUAL AO CICLO NEGATIVO
    OU SEJA: 50% DE DUTY CICLE.
    O CICLO NEGATIVO  COMPARADO COM CICLO POSITIVO E QUALQUER DIFERENA A
    ROTINA IR BLOQUEAR O FUNCIONAMENTO.

    USO DIDTICO (ROTINA EXPERIMENTAL)
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG PICSOURCE

*******************************************************************************/
#include <12F675.h>//pic usado
#device adc=10
#use delay(clock=4000000)//frequ de clock
#fuses WDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT//configurao

//******************************************************************************
//Escolha aqui o modo de operao desejado para cada canal.
//Comente para modo 'reteno' ou descomente para modo 'pulso':

 //      #define canal0_pulso    //canal 0 em modo reteno
       #define canal1_pulso   //canal 1 em modo pulso
       #define canal2_pulso   //canal 2 em modo pulso
       #define canal3_pulso   //canal 3 em modo pulso
 //      #define canal4_pulso   //canal 4 em modo reteno
//******************************************************************************

//bytes
#byte option_reg = 0x81
#byte  trisio = 0x85
#byte  gpio   = 0x05
#byte  t1con  = 0x10
#byte  intcon = 0x0b
#byte  cmcon  = 0x19
#byte  tmr0   = 0x01
#byte  ansel  = 0x9f
#byte  tmr1h   = 0x0f
#byte  tmr1l   = 0x0e

#bit t0ie  =    0x0b.5
#bit t0if  =    0x0b.2
#bit gie   =    0x0b.7
#bit tmr1if =   0x0c.0 //flag overflow timer 1
#bit nTO    =   0x03.4 //status,TO

#bit in_tom   = 0x05.3  //pino 4 - entrada da frequncia
#bit rele0       = 0x05.5  //pino 2 - canal0
#bit rele1       = 0x05.4  //pino 3 - canal1
#bit rele2       = 0x05.2  //pino 5 - canal2
#bit rele3       = 0x05.1  //pino 6 - canal3
#bit rele4       = 0x05.0  //pino 7 - canal4

//variveis globais
#ifndef canal0_pulso
unsigned int1  flag0=0;
#endif
#ifndef canal1_pulso
unsigned int1  flag1=0;
#endif
#ifndef canal2_pulso
unsigned int1  flag2=0;
#endif
#ifndef canal3_pulso
unsigned int1  flag3=0;
#endif
#ifndef canal4_pulso
unsigned int1  flag4=0;
#endif


unsigned int1  flag_t; //flag auxiliar para identificar um reset via WDT
unsigned int8  cnt_del0;
unsigned int8  cnt_del1;
unsigned int8  cnt_del2;
unsigned int8  cnt_del3;
unsigned int8  cnt_del4;
unsigned int16 valfreq;//contador do periodo do sinal de entrada (ciclo positivo)
unsigned int16 valfreq1;//contador do periodo do sinal de entrada (ciclo negativo)

//defines
//Determina as frequncias de operao de cada canal
//Defina limites de frequncia mxima e mnima
//Obs. Rotina testada para valores abaixo de 1 khz. Outros valores acima desta
//frequncia talvez  necessitem de alterao na rotina geradora de frequncia.
//A janela entre a frequncia superior e inferior no deve ser pequena, pois
//como usamos o oscilador interno do PIC, as mudanas de temperatura ambiente
//podem fazer a frequncia ser alterada a ponto de no haver mais comunicao
//entre TX e RX. Valores entre 40 e 50 mostraram ser satisfatrios.

#define PERIOD_MAX0      100000 / 340  //340 HZ CANAL 0 (liga sada de 300 a 340 hz)
#define PERIOD_MIN0      100000 / 300  //300 HZ

#define PERIOD_MAX1      100000 / 470  //470 HZ CANAL 1 (liga sada de 470 a 430 hz)
#define PERIOD_MIN1      100000 / 430  //430 HZ

#define PERIOD_MAX2      100000 / 600  //600 HZ CANAL 2 (liga sada de 570 a 600 hz)
#define PERIOD_MIN2      100000 / 570  //570 HZ

#define PERIOD_MAX3      100000 / 740  //740 HZ CANAL 3 (liga sada de 700 a 740 hz)
#define PERIOD_MIN3      100000 / 700  //700 HZ

#define PERIOD_MAX4      100000 / 848 //848 HZ  CANAL 4 (liga sada de 800 a 848 hz)
#define PERIOD_MIN4      100000 / 800  //800 HZ

#define DIFERENCA_MAX    6  // diferena maxima entre capturas
                             // no ciclo positivo e negativo do tom
#define VALOR_DESLG     25  //valor usado no desligamento de rele em modo pulso

//sub-rotinas
/*******************************************************************************
     ROTINA DE COMPARAO DA FREQUENCIA CAPTURADA COM A FAIXA DE ATUAO
     
     1) Se canal est definido como modo pulso:
     Compara se valor capturado est dentro da janela de atuao. Se estiver
     ento aciona o respectivo rele do canal e zera contador de tempo de desli-
     gamento da sada. Quando a contagem chega a zero desligar o rele do canal.
     Tambm se houver um reset pelo WDT tambm ser desligados todos canais em
     modo pulso.
     2) Se canal est definido como modo reteno:
     Compara se valor capturado est dentro da janela de atuao. Se estiver
     zera contador de inverso (cnt_deslx). Se o flagX (onde X= 0,1,2,3 ou 4)
     estiver resetado, ento permitir que ocorra a inverso da sada deste
     canal. Nesta ocasio, ser setado o flagX para no ocorrer novas inverses
     at que o contador conte o valor estipulado (sem ser zerado).
     
*******************************************************************************/

void compara_freq(){

 //Compara algum valor capturado e compara com as faixas limites de cada canal
    #ifdef canal0_pulso
    if(valfreq >= PERIOD_MAX0 && valfreq <= PERIOD_MIN0) {rele0=1; cnt_del0=0;}
    if(rele0==1){
      cnt_del0++;
      if((cnt_del0>VALOR_DESLG) || (flag_t==1)){rele0=0;}//se chegou na contagem
                                                         //ou houve reset do WTD
    }
    #else
    if(valfreq >= PERIOD_MAX0 && valfreq <= PERIOD_MIN0){
      cnt_del0=0;
      if(!flag0){ rele0=!rele0; flag0=1; }
    }
    cnt_del0++;
    if(cnt_del0>VALOR_DESLG){flag0=0; cnt_del0--;}
    #endif


    #ifdef canal1_pulso
    if(valfreq >= PERIOD_MAX1 && valfreq <= PERIOD_MIN1) {rele1=1; cnt_del1=0;}
    if(rele1==1){
      cnt_del1++;
      if((cnt_del1>VALOR_DESLG) || (flag_t==1)){rele1=0;}
    }
    #else
    if(valfreq >= PERIOD_MAX1 && valfreq <= PERIOD_MIN1){
      cnt_del1=0;
      if(!flag1){ rele1=!rele1; flag1=1; }
    }
    cnt_del1++;
    if(cnt_del1>VALOR_DESLG){flag1=0; cnt_del1--;}
    #endif


    #ifdef canal2_pulso
    if(valfreq >= PERIOD_MAX2 && valfreq <= PERIOD_MIN2) {rele2=1; cnt_del2=0;}
    if(rele2==1){
      cnt_del2++;
      if((cnt_del2>VALOR_DESLG) || (flag_t==1)){rele2=0;}
    }
    #else
    if(valfreq >= PERIOD_MAX2 && valfreq <= PERIOD_MIN2){
      cnt_del2=0;
      if(!flag2){ rele2=!rele2; flag2=1; }
    }
    cnt_del2++;
    if(cnt_del2>VALOR_DESLG){flag2=0; cnt_del2--;}
    #endif


   #ifdef canal3_pulso
    if(valfreq >= PERIOD_MAX3 && valfreq <= PERIOD_MIN3) {rele3=1; cnt_del3=0;}
    if(rele3==1){
      cnt_del3++;
      if((cnt_del3>VALOR_DESLG) || (flag_t==1)){rele3=0;}
    }
    #else
    if(valfreq >= PERIOD_MAX3 && valfreq <= PERIOD_MIN3){
      cnt_del3=0;
      if(!flag3){ rele3=!rele3; flag3=1; }
    }
    cnt_del3++;
    if(cnt_del3>VALOR_DESLG){flag3=0; cnt_del3--;}
    #endif


   #ifdef canal4_pulso
    if(valfreq >= PERIOD_MAX4 && valfreq <= PERIOD_MIN4) {rele4=1; cnt_del4=0;}
    if(rele4==1){
      cnt_del4++;
      if((cnt_del4>VALOR_DESLG) || (flag_t==1)){rele4=0;}
    }
    #else
    if(valfreq >= PERIOD_MAX4 && valfreq <= PERIOD_MIN4){
      cnt_del4=0;
      if(!flag4){ rele4=!rele4; flag4=1; }
    }
    cnt_del4++;
    if(cnt_del4>VALOR_DESLG){flag4=0; cnt_del4--;}
    #endif

}
/*******************************************************************************
   ROTINA DE CAPTURA DE FREQUNCIAS NA ENTRADA (SEQUENCIALMENTE TRANSMITIDAS)
   
   Reseta flag_t (cpia do status,TO)
   Zera contador de captura de ciclo negativo e positivo.
   Sincroniza a rotina com o sinal a medir na entrada pino 4.
   Captura semi-ciclo positivo do sinal.
   Captura semi-ciclo negativo do sinal.
   Compara se tem a mesma durao (com pequena tolerncia definida pelo valor
   da constante 'DIFERENCA_MAX'.
   Se diferentes, cancela a leitura zerando os contador cnt_freq e cnt_freq1.
   Se iguais, soma os 2 contadores e coloca resultado em cnt_freq para futura
   comparao na rotina 'compara_freq()'.
   Caso o sinal da entrada seja removido ou no h nenhuma frequncia, o timer
   WDT ir resetar o programa e sair de um dos laos 'while' (sincronizao ou
   captura de valores da frequncia de entrada).
              
*******************************************************************************/

void captura_freq(){
  //inicia captura do valor da frequncia de entrada (no pino 4) 
      flag_t=0;//reseta flag copia de status,TO

      valfreq=0; //zera contador do ciclo positivo
      valfreq1=0;//zera contador do ciclo negativo
      
      while(in_tom); //aguarda at terminar ciclo positivo (sincroniza sinal).
                     //se no tiver sinal, o WDT ir resetar o programa aps 18ms
      while(!in_tom);//aguarda at terminar negativo (sincroniza sinal).
                     //se no tiver sinal, o WDT ir resetar o programa aps 18ms

      while(in_tom){//conta 1 unidade a cada 10 us.
      //se no tiver sinal na entrada, o WDT ir resetar o programa aps 18ms
       valfreq++;
       delay_us(3);
      }

      while(!in_tom) {//conta 1 unidade a cada 10 us
        //se no tiver sinal na entrada, o WDT ir resetar o programa aps 18ms
        valfreq1++;
        delay_us(3);
      }
        
    //verifica erro de captura que possa acionar aleatriamente um canal errado
      if((valfreq1>valfreq +DIFERENCA_MAX)||(valfreq>valfreq1+DIFERENCA_MAX)){
          valfreq=0;//erro foi detectado: cancela acionamento
          valfreq1=0;
      }
      else{
         valfreq=valfreq+valfreq1; //ok, soma os valores
      }

}
/*******************************************************************************
                        ROTINA MAIN
*******************************************************************************/

void main() {
   ansel = 0;//desliga conversor AD
   option_reg = 0;//watch dog /1 (18 ms de tempo antes do reset)
   trisio = 0b001000;//ajusta entradas e sadas
   cmcon = 7;//desliga comparadores
   if(nTO) {//verifica se reset foi inicial ou pelo WDT
     gpio = 0;//zera tudo somente no reset inicial (ao ligar alimentao)
     valfreq=0;
     cnt_del0=0;
     cnt_del1=0;
     cnt_del2=0;
     cnt_del3=0;
     cnt_del4=0;     
   }
   else{//reset pelo WDT (WatchDog Timer)
     flag_t=1;//seta flag auxiliar
   }

/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/

   for(;;){  
    restart_wdt();//restart o WDT
    compara_freq();//compara valores e aciona reles se iguais
    captura_freq();//captura frequncias de acionamento na entrada pino 4 
   }//for(;;)

}//main


/*******************************************************************************
                           FIM DO PROGRAMA
*******************************************************************************/

